/*
                    PROGRAMMA DI SIMULAZIONE MICROCALCOLATORE
                    
AUTORE: Giancarlo Perlo, Mauro Puccinelli
DATA : 20/3/2013
ULTIMA REVISIONE: 16/03/2015

DESCRIZIONE:
            
Questo programma simula il funzionamento di un microcalcolatore a 8 bit con 
RAM 256 x 8 (questi parametri possono essere modificati agendo sulle costanti
ADDR_BIT e WORD_BIT definite in testa al programma).                    

Il microcalcolatore  fornito anche di 4 set di 8 interruttori di ingresso
(numerati da 0 a 3) per l'input dei dati e di 4 set di 8 led di uscita
per l'output dei dati.

Il microprocessore ha 4 registri interni (A, B, C e D) e due bit di flag
per il risultato delle operazioni matematiche (S=0 segno positivo,
S=1 segno negativo, Z=0 valore diverso da zero, Z=1 valore uguale a zero).

Il programma consente di:
             
1) Scrivere le istruzioni del programma nella RAM a partire dall'indirizzo 0             
2) Eseguire il programma-macchina step by step a partire dall'indirizzo 0 finch non viene incontrata una HALT
3) Modificare il contenuto di una specifica locazione di memoria
4) Visualizzare il contenuto dell'intera memoria
5) Salvare il programma su un file binario
6) Caricare il programma da un file binario
7) Settare il valore degli switch di ingresso

Durante l'esecuzione del programma-macchina ad ogni step vengono visualizzati i valori
di tutti i registri interni, dei flag e dei dispositivi di input e output.

*/

#include <cstdlib>
#include <iostream>
#include <conio.h>
#include <math.h>
#include <fstream>
#include <iomanip>
#include <sstream>

//                                      INSTRUCTION SET
// *******************************************************************************************    
//Label             //Descrizione                               // Codice operativo (binario)
#define HALT 0      // Ferma il programma                        0
#define LDAN 1      // A = n                                     1
#define LDBN 2      // B = n                                     10
#define LDCN 3      // C = n                                     11
#define LDDN 4      // D = n                                     100
#define DECA 5      // A = A - 1                                 101
#define INCA 6      // A = A + 1                                 110
#define ADDAB 7     // A = A + B                                 111
#define ADDAC 8     // A = A + C                                 1000
#define ADDAD 9     // A = A + D                                 1001
#define SUBAB 10    // A = A - B                                 1010
#define SUBAC 11    // A = A - C                                 1011
#define SUBAD 12    // A = A - D                                 1100
#define JPN 13      // Salta a n                                 1101
#define JPPN 14     // Salta a n se positivo                     1110
#define JPNN 15     // Salta a n se negativo                     1111
#define JPZN 16     // Salta a n se  zero                        10000
#define JPNZN 17    // Salta a n se non zero                     10001
#define LDANN 18    // Carica in A il contenuto dell'indirizzo n 10010
#define LDNNA 19    // Carica all'indirizzo n il contenuto di A  10011
#define LDAB 20     // A = B                                     10100
#define LDAC 21     // A = C                                     10101
#define LDAD 22     // A = D                                     10110
#define LDBA 23     // B = A                                     10111
#define LDCA 24     // C = A                                     11000
#define LDDA 25     // D = A                                     11001
#define INPA 27     // Carica in A il set di switch di ingressi  11011
#define OUTA 28     // Invia A sul set di led di uscita          11100
#define LDABB 29    // Carica in A il contenuto dell'indirizzo B 11101
#define LDBBA 30    // Carica all'indirizzo B il contenuto di A  11110
#define LDACC 31    // Carica in A il contenuto dell'indirizzo C 11111
#define LDCCA 32    // Carica all'indirizzo C il contenuto di A  100000
#define LDADD 33    // Carica in A il contenuto dell'indirizzo D 100001
#define LDDDA 34    // Carica all'indirizzo D il contenuto di A  100010
#define DECB 35      // B = B - 1                                100011
#define INCB 36      // B = B + 1                                100100
#define DECC 37      // C = C - 1                                100101
#define INCC 38      // C = C + 1                                100110
#define DECD 39      // D = D - 1                                100111
#define INCD 40      // D = D + 1                                101000
#define LDEN 41       // E = n                                   101001
#define ADDAE 42     // A = A + E                                101010
#define SUBAE 43      // A = A - E                               101011
#define LDEA 44      // E = A                                    101100
#define DECE 45      // E = E - 1                                101101
#define INCE 46      // E = E + 1                                101110
#define LDAEE 47    // Carica in A il contenuto dell'indirizzo E 101111
#define LDEEA 48    // Carica all'indirizzo E il contenuto di A  110000
#define LDAE 49     // A = E                                     110001
#define ANDAN 50   // A & n                                      110010
#define ORAN 51    // A | n                                      110011
#define NOTA 52    // !A                                         110100
#define SRLA 53    // SRL A                                      110101
#define SLLA 54    // SLL A                                      110110
#define CPAN 26     // Confronta A con  n e setta i flag          11010
#define CPAC 55     // Confronta A con C e setta i flag          110111
#define CPAD 56     // Confronta A con D e setta i flag          111000
#define CPAE 57     // Confronta A con E e setta i flag          111001
#define CPAB 58     // Confronta A con B e setta i flag          111010

// ******************************************************************************************

// Macro per assegnare i valori ai flag in base al risultato dell'ultima operazione eseguita su A
#define set_flags(X) S= (X>=0) ? 0:1; Z = (X!=0) ? 0:1;

// Bit di indirizzo della RAM
#define ADDR_BIT 8

// Bit della parola della RAM
#define WORD_BIT 8

// Dimensione (in parole) della RAM usata
#define DIM_RAM (int) pow(2,ADDR_BIT)

// Massimo valore memorizzabile in ogni singola parola RAM
#define DIM_PAROLA (int) pow(2,WORD_BIT)

int word_check(int); //Controlla correttezza parola
int addr_check(int); // Controlla correttezza indirizzo

// converte il numero decimale passato come argomento in un valore binario di ritorno
int converti_bin(int num);

// converte il numero binario passato come argomento in un valore decimale intero di ritorno
int converti_dec(int num);

// Verifica se un numero  binario oppure no
int verifica_bin(int num);

// Calcola il complemento a 2 di un numero binario
int complementa2(int bin);

// Esegue l'and bit a bit
int andd(int a, int b);

// Esegue l'or bit a bit
int orr(int a, int b);

// Esegue il not dei singoli bit
int nott(int a);

// Controlla se una stringa  un numero valido nella base nNumberBase
bool isNumeric( char* pszInput, int nNumberBase );

// Avvia il timer
void avvia_timer(void);

// Calcola i secondi trascorsi dall'avvio del timer
double ferma_timer(void);

using namespace std;

int main(int argc, char *argv[])
{
    
    int      ram[DIM_RAM],          // Memoria RAM contenente il programma da eseguire e i dati
             scelta=-1,             // Variabile scelta menu principale
             instr,                 // Codice operativo istruzione da eseguire
             pc=0, pcc,             // Program Counter e sua copia
             A=0, B=0, C=0, D=0,E=0, // Registri
             S=0, Z=0,              // Flag: S=segno, Z=zero
             inp[4], out[4],        // Switch di ingresso e led di uscita
             stop, addr, val, tmp;

        
    char conferma, programma[40];
    char instr1[WORD_BIT+1];
    
    ofstream output; 
    ifstream input; 

    //Pulisce RAM
    for (int i=0;i <DIM_RAM;i++)
        ram[i]=0;

    // Azzera i dispositivi di I/O
    for (int i=0 ; i <4; i++)
        {
        inp[i] = 0;
        out[i] = 0;
        }

    while (scelta!=9)
          {
          cout<<"\nMENU PRINCIPALE\n\n";
          cout<<"1) Scrivi programma\n";
          cout<<"2) Esegui step by step\n";
          cout<<"3) Esegui no stop\n";          
          cout<<"4) Scrivi in memoria\n";
          cout<<"5) Mostra contenuto memoria\n";
          cout<<"6) Salva programma\n";
          cout<<"7) Carica programma\n";
          cout<<"8) Setta gli switch di ingresso\n";
          cout<<"9) Esci\n";
          cout<<"\nScegli: ";
          cin>>scelta;
          
          switch (scelta)
                 {
                         
                 case 1:

                      pcc = 0; // Inizializzo la copia del PC all'indirizzo prima locazione RAM
                      
                      cout<<"\n\nInserisci il codice (valore non valido per terminare)\n\n";

                      // Scrive il programma in memoria fino al numero masssimo di locazioni disponibili                    
                      cout<<"Addr\tInstr\n"; 
                      while (pcc<DIM_RAM)
                            {
                                          
                                        
                            cout<<converti_bin(pcc)<<"\t";
                            cin>>instr1; // codice operativo istruzione o valore operandi
                            
                            instr = atoi(instr1); //converte il codice operativo in numero
                            
                            if (!strcmp(instr1,"STOP") || !strcmp(instr1,"stop") || 
                            !isNumeric( instr1, 10) || !word_check(instr))
                               {
                               cout<<"\nSTOP o Codice non valido. Vuoi terminare (s/n)? ";
                               cin>> conferma;
                               if (conferma=='s' || conferma=='S')
                                  break;
                               else
                                   continue;
                               }
                               
                            ram[pcc] = converti_dec(instr);
                            pcc++;
                            }
                      break;
                 
                 case 2:
                 case 3:
                 
                      avvia_timer();
                      stop=1;
                      
                      // Ciclo di scansione memoria ed esecuzione programma
                      for (pc=0;stop!=0 && pc<DIM_RAM;pc++)
                          {
                          
                          if (ferma_timer()>2 && scelta==3)
                             {
                             cout<<"\n\n********** ERRORE: IDENTIFICATO CICLO INFINITO\n********** ESECUZIONE ABORTITA\n\n";
                             break;
                             }
                          
                          if (scelta==2)
                            {
                           cout<<"\nPC = "<<setfill('0') << setw(ADDR_BIT) <<converti_bin(pc)<<
                            "\t\tCode = "<<setfill('0') << setw(WORD_BIT) <<converti_bin(ram[pc])<<"\t\t"<<"SZ = "<<S<<Z<<"\n";
                            cout<<"A = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(A)<<
                            "\t\tB = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(B)<<
                            "\t\tC = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(C)<<
                            "\nD = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(D)<<
                            "\t\tE = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(E)<<"\n";     

                            for (int i=0; i<4; i++)
                              cout <<"inp"<<i<<" = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(inp[i])<<"\t";
                            cout<<"\n";
                            for (int i=0; i<4; i++)
                              cout <<"out"<<i<<" = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(out[i])<<"\t";
                            cout<<"\n";
                                       
                            }
                          
                          // Selezione istruzione da eseguire
                          switch (ram[pc])
                                 {
                                 case LDAN:
                                      // LD A, n
                                      A = ram[++pc];
                                      break;

                                 case LDEN:
                                      // LD A, n
                                      E = ram[++pc];
                                      break;

                                      
                                 case LDBN:
                                      // LD B, n
                                      B = ram[++pc];
                                      break;
                                      
                                 case LDCN:
                                      // LD C, n
                                      C = ram[++pc];
                                      break;
                                      
                                 case LDDN:
                                      // LD D, n
                                      D = ram[++pc];
                                      break;
                                      
                                 case DECA:
                                      // DEC A
                                      A--;
                                      
                                     set_flags(A);
                                         
                                      break;
                                      
                                 case INCA:
                                      // INC A
                                      A++;
                                      
                                     set_flags(A);                                     
                                      break;
                                      
                                 case ADDAB:
                                      // ADD A,B
                                      A = A + B;

                                     set_flags(A);
                                     
                                      break;
                                    
                                 case ADDAC:
                                      // ADD A,C
                                      A = A + C;

                                     set_flags(A);
                                      break;
                                      
                                 case ADDAD:
                                      // ADD A,D
                                      A = A + D;

                                     set_flags(A);
                                      break;
                                      
                                 case ADDAE:
                                      // ADD A,E
                                      A = A + E;

                                     set_flags(A);
                                      break;
                                      
                                 case SUBAB:
                                     // SUB A,B
                                      A = A - B;
                                     set_flags(A);
                                    
                                      break;

                                 case SUBAC:
                                     // SUB A,C
                                      A = A - C;

                                     set_flags(A);
                                      break;                                       

                                 case SUBAD:
                                     // SUB A,D
                                      A = A - D;

                                     set_flags(A);
                                      break;                                       
                                 
                                 case SUBAE:
                                     // SUB A,E
                                      A = A - E;

                                     set_flags(A);
                                      break;   
                                                                                                                          
                                 case JPN:
                                      // JP n
                                      pc = ram[++pc]-1;
                                      break;
                                      
                                 case JPPN:
                                      // JP P, n
                                      if (S==0)
                                           pc = ram[++pc]-1;
                                      else
                                          pc++;
                                      break;
                                  
                                 case JPNN:
                                      // JP N, n
                                      if (S==1)
                                           pc = ram[++pc]-1;
                                      else
                                          pc++;
                                      break;
                                         
                                 case JPZN:
                                      // JP Z, n
                                      if (Z==1)
                                           pc = ram[++pc]-1;
                                      else
                                          pc++;                                          
                                      break;
                                      
                                 case JPNZN:
                                      // JP Z, n
                                      if (Z==0)
                                           pc = ram[++pc]-1;
                                      else
                                          pc++;
                                      break;

                                 case LDANN:
                                      // LD A,(n)
                                      A = ram[ram[++pc]];

                                      break;
                                      
                                 case LDNNA:
                                      // LD (n), A
                                      ram[ram[++pc]] = A;
                                      break;
                                 
                                 case LDAB:
                                      // LD A, B
                                      A = B;
                                      break;
                                                                            
                                 case LDAC:
                                      // LD A, C
                                      A = C;
                                      break;
                                      
                                 case LDAD:
                                      // LD A, D
                                      A = D;
                                      break;
                                      
                                 case LDAE:
                                      // LD A, E
                                      A = E;
                                      break;
                                      
                                 case CPAB:
                                      //Confronta A con B e setta i flag
                                     tmp = A - B;
                                     set_flags(tmp);
                                     break;
                                     
                                 case CPAN:
                                      //Confronta A con un numero e setta i flag
                                     tmp = A - ram[++pc];
                                     set_flags(tmp);
                                     break;

                                 case CPAC:
                                      //Confronta A con C e setta i flag
                                     tmp = A - C;
                                     set_flags(tmp);
                                     break;
                                     
                                 case CPAD:
                                      //Confronta A con D e setta i flag
                                     tmp = A - D;
                                     set_flags(tmp);
                                     break;
                                     
                                 case CPAE:
                                      //Confronta A con E e setta i flag
                                     tmp = A - E;
                                     set_flags(tmp);
                                     break;
                                     
                                 case INPA:
                                      // A = INP(n)
                                      tmp = ram[++pc];
                                      if (converti_dec(tmp)<0 || converti_dec(tmp)>3)
                                         {
                                         cout<<"\n\nFATAL ERROR: valore di input non valido!\n";
                                         stop=0;                                                           
                                       }
                                      A = inp[tmp];   
                                      break;

                                 case OUTA:
                                      // OUT(n) = A
                                      tmp = ram[++pc];
                                      if (converti_dec(tmp)<0 || converti_dec(tmp)>3)
                                         {
                                         cout<<"\n\nFATAL ERROR: valore di output non valido!\n";
                                         stop=0;                                                           
                                       }
                                      out[tmp] = A;   
                                      break;                                                                            
                                     
                                 case HALT:
                                      // HALT
                                      stop=0;
                               
                                      break;   
                                      
                                 case LDBA:
                                      // LD B, A
                                      B = A;
                                      break;
                                                                                                         
                                 case LDCA:
                                      // LD C, A
                                      C = A;
                                      break;

                                 case LDDA:
                                      // LD D, A
                                      D = A;
                                      break;
                                      
                                 case LDEA:
                                      // LD E, A
                                      E = A;
                                      break;
                                      
                                 case LDABB:
                                      // LD A,(B)
                                      A = ram[B];
                                      break;
                                      
                                 case LDBBA:
                                      // LD (B), A
                                      ram[B] = A;
                                      break;
                                      
                                 case LDACC:
                                      // LD A,(C)
                                      A = ram[C];
                                      break;
                                      
                                 case LDCCA:
                                      // LD (C), A
                                      ram[C] = A;
                                      break;

                                 case LDADD:
                                      // LD A,(D)
                                      A = ram[D];
                                      break;
                                      
                                 case LDDDA:
                                      // LD (D), A
                                      ram[D] = A;
                                      break;                                

                                 case LDAEE:
                                      // LD A,(E)
                                      A = ram[E];
                                      break;
                                      
                                 case LDEEA:
                                      // LD (E), A
                                      ram[E] = A;
                                      break;                                

                                     
                                 case DECB:
                                      // DEC B
                                      B--;
                                      
                                     set_flags(B);
                                         
                                      break;
                                      
                                 case INCB:
                                      // INC B
                                      B++;
                                      
                                     set_flags(B);                                     
                                      break;

                                     
                                 case DECC:
                                      // DEC C
                                      C--;
                                      
                                     set_flags(C);
                                         
                                      break;
                                      
                                 case INCC:
                                      // INC C
                                      C++;
                                      
                                     set_flags(C);                                     
                                      break;
                                      
                                     
                                 case DECD:
                                      // DEC D
                                      D--;
                                      
                                     set_flags(D);
                                         
                                      break;
                                      
                                 case INCD:
                                      // INC D
                                      D++;
                                      
                                     set_flags(D);                                     
                                      break;

                                 case DECE:
                                      // DEC E
                                      E--;
                                      
                                     set_flags(E);
                                         
                                      break;
                                      
                                 case INCE:
                                      // INC E
                                      E++;
                                      
                                     set_flags(E);                                     
                                      break;
                                      
                                 case ANDAN:
                                      // And bit a bit
                                      A = converti_dec(andd(converti_bin(A),converti_bin(ram[++pc])));
                                      set_flags(A);
                                      break;

                                 case ORAN:
                                      // Or bit a bit
                                      A = converti_dec(orr(converti_bin(A),converti_bin(ram[++pc])));
                                      set_flags(A);
                                      break;

                                 case NOTA:
                                      // Inversione logica
                                      A = converti_dec(nott(converti_bin(A)));
                                      set_flags(A);
                                      break;
                                      
                                 case SRLA:
                                      // Shift verso destra
                                      A = converti_bin(A)/10;
                                      A = converti_dec(A);
                                      break;
                                      
                                 case SLLA:
                                      // Shift verso sinistra
                                      A = converti_bin(A)*10;
                                      A = converti_dec(A);
                                      break;
                                      
                                 default:
                                         cout<<"\n\nFATAL ERROR: codice operativo istruzione non riconosciuto!\n";
                                         stop=0;
                                         break;

                                         
                                 }              
                          if (scelta==2)
                              {
                              cout<<"\nPremi un tasto per continuare oppure 0 per terminare.... ";              
                              if (getch()=='0')
                                    stop = 0;
                            }             
                          }
                          
                        if (scelta==3)
                            {
                            cout<<"\nPC = "<<setfill('0') << setw(ADDR_BIT) <<converti_bin(pc-1)<<
                            "\t\tCode = "<<setfill('0') << setw(WORD_BIT) <<converti_bin(ram[pc-1])<<"\t\t"<<"SZ = "<<S<<Z<<"\n";
                            cout<<"A = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(A)<<
                            "\t\tB = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(B)<<
                            "\t\tC = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(C)<<
                            "\nD = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(D)<<
                            "\t\tE = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(E)<<"\n";     
                            
                            for (int i=0; i<4; i++)
                              cout <<"inp"<<i<<" = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(inp[i])<<"\t";
                            cout<<"\n";
                            for (int i=0; i<4; i++)
                              cout <<"out"<<i<<" = "<<setfill('0') << setw(WORD_BIT)<<converti_bin(out[i])<<"\t";
                            cout<<"\n";
                            }
  
                          
                          break;
                  case 4:
                         cout<<"\nIndirizzo base (0-"<<converti_bin(DIM_RAM-1)<<"): ";
                         cin>>addr;
                         if (!addr_check(addr))
                            {
                            cout<<"\nERRORE: indirizzo base non valido!\n";
                            break;
                            }
                       
                                            
                       while (1)
                         {
                         cout<<"\nIndirizzo: "<<addr<<"\n";    
                         cout<<"Valore ("<<converti_bin(-DIM_PAROLA/2)<<" - "<<converti_bin(DIM_PAROLA/2-1)<<", non valido per terminare): ";
                         cin>>val;
                         if (!word_check(val))
                            break;
                         ram[converti_dec(addr)]=converti_dec(val);
                         addr = converti_bin(converti_dec(addr)+1);
                         }
                       
                       break;
                       
                  case 5:
                       cout<<"\nAddr\tWord\n\n";
                       for (int i=0;i <DIM_RAM;i=i+4)
                           {
                           cout<< setfill('0') << setw(ADDR_BIT) <<converti_bin(i);
                           for (int j=0; j<4; j++)
                               cout<< "\t"<<setfill('0') << setw(WORD_BIT) <<converti_bin(ram[i+j]);
//                           cout<<converti_bin(i)<<"\t\t"<<converti_bin(ram[i])<<"\n";
                             cout<<"\n";
                             }

                       break;
                           
                  case 6:
                       cout<<"\nFornisci il nome del file su cui salvare: ";
                       cin>>programma;
                       output.clear();
                       output.open(programma, ios::out);
                       for (int i=0;i<pcc;i++)
                            output<<converti_bin(ram[i])<<endl;
                       output.close();
                       break;
                       
                  case 7:
                       cout<<"\nFornisci il nome del file da caricare: ";
                       cin>>programma;
                       input.clear(); //necessario altrimenti non ricarica il file
                       input.open(programma, ios::in );                                       
                       for (int i=0;i<DIM_RAM;i++)
                            {
                            input>>tmp;
                            if (input.eof()==1)
                                break;                            
                            ram[i] = converti_dec(tmp);
                            }
                       input.close();
                       break;
                      
                  case 8:
                       do
                       {
                       cout<<"\nNumero del dispositivo di input (0-3): ";
                       cin>>tmp;
                       }
                       while (tmp<0 || tmp>3);
                       
                       do
                       {
                       cout<<"Valore ("<<converti_bin(-DIM_PAROLA/2)<<" - "<<converti_bin(DIM_PAROLA/2-1)<<"): ";
                       cin>>val;
                       inp[tmp] = converti_dec(val);
                       }
                       while (!word_check(val));
                       break;
                       
                  default:
                       cout<<"\n\nSelezione non valida!";
                       break;
                  }
          
          
          }

    return EXIT_SUCCESS;
}


// Controllo correttezza indirizzo
int addr_check(int x)
{
    if (!verifica_bin(x))
       return 0; 
       

    if (x<0 || x>converti_bin(DIM_RAM))
       return 0;

    return 1;
}

// Controllo correttezza parola
int word_check(int x)
{
    if (!verifica_bin(x))
       return 0;
       
    if (converti_dec(x)<-DIM_PAROLA/2 || converti_dec(x)>DIM_PAROLA/2-1)
       return 0;
    return 1;
}

/*
converte il numero decimale passato come argomento in un valore binario di ritorno
*/
int converti_bin(int num)
{


int vet[30],i, nega=0;   
int ris=0, pot=1;

// Se il numero  negativo
if (num<0)
   {
   //rende positivo il numero
   num = -num;
   //setta il flag
   nega=1;
   }
   
for (i=0; ; i++)
	{
	
	vet[i] = num%2;
	num = num/2;
	if (num==0)
		break;  
	pot = pot *10;
	}

for (;i>=0;i--)    
	{
	ris = ris + pot*vet[i];
	pot = pot/10;
	}

	
if (nega)
   ris = complementa2(ris);

   	
return ris;
}

/*
Converte il numero binario passato come argomento in un valore decimale intero di ritorno
*/
int converti_dec(int num)
{
int pot=1, cifra, ris=0, nega=0;

      
      
if (num/(int)pow(10,WORD_BIT-1) == 1)
   {
   nega = 1;
   num = complementa2(num);
}

for (;;)
	{
	cifra = (int) (num%10);    
	ris = ris + cifra*pot;
	num = num/10;
	if (num==0)
		break;
	pot = pot*2;
	}
	
if (nega)
   ris = -ris;
   
return ris;
}

/*
Verifica se un numero  binario oppure no.
La funzione torna uno se l'argomento  un numero binario, torna zero altrimenti.
*/
int verifica_bin(int num)
{
 int i, cifra;
 for (i=0; ; i++)
	{
	
	cifra = num%10;
	if (cifra!=0 && cifra!=1)
	   return 0;
	num = num/10;
	if (num==0)
		break;  
  }
 return 1;
    
}

int complementa2(int bin)
{
int cifre[WORD_BIT]; //vettore contenente le cifre del numero
int i, ris=0, pot=1;

//Azzero il vettore
for (i=0;i<WORD_BIT;i++)
    cifre[i] = 0;

//Estraggo le cifre dal numero    
for (i=0;i<WORD_BIT;i++)
    {
    cifre[i] = (bin%10)==0?1:0;
    bin = bin/10;
    }


//Sommo uno
for (i=0; i<WORD_BIT;i++)
    {
    if (cifre[i]==0)
       {
        cifre[i] = 1;
        break;
        }
    cifre[i]=0;       
}
// Ricompongo il numero
for (i=0;i<WORD_BIT;i++)
    {
    ris = ris + cifre[i]*pot;
    pot = pot*10;
    }


return ris;    
}

// AND bit a BIT
int andd(int a, int b)
{
int ris = 0, cif, pot = 1;

while (a!=0 && b!=0)
      {
      if (a%10==1 && b%10 ==1)
         cif = 1;
      else
          cif = 0;
      ris = ris + pot*cif;
      pot = pot*10;
      a = a/10;
      b = b/10;
      }
return ris;
}

// OR bit a BIT
int orr(int a, int b)
{
int ris = 0, cif, pot = 1;

while (a!=0 || b!=0)
      {
      if (a%10==0 && b%10 ==0)
         cif = 0;
      else
          cif = 1;
      ris = ris + pot*cif;
      pot = pot*10;
      a = a/10;
      b = b/10;
      }
return ris;
}

// NOT
int nott(int a)
{
int ris = 0, cif, pot = 1;

for (int i=0; i <WORD_BIT; i++)
      {
      if (a%10==0)
         cif = 1;
      else
          cif = 0;
      ris = ris + pot*cif;
      pot = pot*10;
      a = a/10;
      }
return ris;
}

bool isNumeric( char* pszInput, int nNumberBase )
{
	istringstream iss( pszInput );
 
	if ( nNumberBase == 10 )
	{
		double dTestSink;
		iss >> dTestSink;
	}
	else if ( nNumberBase == 8 || nNumberBase == 16 )
	{
		int nTestSink;
		iss >> ( ( nNumberBase == 8 ) ? oct : hex ) >> nTestSink;
	}
	else
		return false;
 
	// was any input successfully consumed/converted?
	if ( ! iss )
		return false;
 
	// was all the input successfully consumed/converted?
	return ( iss.rdbuf()->in_avail() == 0 );
}

#include <time.h>

// Variabili globali:
time_t cstart ;

// Avvia il timer interno al PC
void avvia_timer(void)
{
 cstart = clock();  
}


// Torna il tempo (in secondi) trascorso dal momento della chiamata
// alla funzione "avvia_timer"
double ferma_timer(void)
{
time_t cend ;

cend = clock(); 	

/* Visualizza il numero di secondi passati */
return ((double)cend - cstart) / CLK_TCK ;

}
